package ifneeded shtmlview::shtmlview 1.1.2 [list source [file join $dir shtmlview.tcl]]
package ifneeded shtmlview::doctools    0.1 [list source [file join $dir shtmlview-doctools.tcl]]
package ifneeded shtmlview::mkdoc       0.1 [list source [file join $dir shtmlview-mkdoc.tcl]]
